#!perl

BEGIN {
    unless ($ENV{AUTHOR_TESTING}) {
        require Test::More;
        Test::More::plan(skip_all =>
                         'these tests are for testing by the author');
    }
}

use strict;
use warnings;

use Test::More tests => 20605;

###############################################################################
# Read and load configuration file and backend library.

my $conffile = 't/author-lib-meta-config.conf';
open CONFFILE, $conffile or die "$conffile: can't open file for reading: $!";
my $confdata = do { local $/ = undef; <CONFFILE>; };
close CONFFILE or die "$conffile: can't close file after reading: $!";

our ($LIB, $REF);
eval $confdata;
die $@ if $@;

eval "require $LIB";
die $@ if $@;

###############################################################################

can_ok($LIB, '_zeros');

use lib 't';
use Math::BigInt::Lib::TestUtil qw< randstr >;

# Generate test data.

my @data;

for (my $p = 0 ; $p <= 100 ; ++ $p) {
    for (my $q = 0 ; $q <= 50 ; ++ $q) {
        my $in0  = randstr($p, 10)          # p digit number (base 10)
                 . (1 + int rand 9)         # ensure non-zero digit
                 . ("0" x $q);              # q zeros
        my $out0 = $q;
        push @data, [ $in0, $out0 ];
    }
}

# List context.

for (my $i = 0 ; $i <= $#data ; ++ $i) {
    my ($in0, $out0) = @{ $data[$i] };

    my ($x, @got);

    my $test = qq|\$x = $LIB->_new("$in0"); | .
               qq|\@got = $LIB->_zeros(\$x);|;

    eval $test;
    is($@, "", "'$test' gives emtpy \$\@");

    subtest "_zeros() in list context: $test", sub {
        plan tests => 3,

        cmp_ok(scalar @got, "==", 1,
               "'$test' gives one output arg");

        is(ref($got[0]), "",
           "output arg is a Perl scalar");

        cmp_ok($got[0], "==", $out0,
               "output arg has the right value");
    };
}

# Scalar context.

for (my $i = 0 ; $i <= $#data ; ++ $i) {
    my ($in0, $out0) = @{ $data[$i] };

    my ($x, $got);

    my $test = qq|\$x = $LIB->_new("$in0"); | .
               qq|\$got = $LIB->_zeros(\$x);|;

    eval $test;
    is($@, "", "'$test' gives emtpy \$\@");

    subtest "_zeros() in scalar context: $test", sub {
        plan tests => 2,

        is(ref($got), "",
           "output arg is a Perl scalar");

        cmp_ok($got, "==", $out0,
               "output arg has the right value");
    };
}
